import requests
import unittest


@unittest.skip
class RequestTest(unittest.TestCase):
    def test_get_returns_200(self):
        r = requests.get('https://jsonplaceholder.typicode.com/todos/1')
        self.assertEqual(r.status_code, 200)

    def test_get_with_params_returns_json(self):
        payload = {
            'postId' : 1
        }
        r = requests.get('https://jsonplaceholder.typicode.com/comments', params=payload)
        self.assertEqual(len(r.json()), 5)

    def test_get_with_params_returns_json_with_expected_attributes(self):
        expected_post_id = 1
        payload = {
            'postId': expected_post_id
        }
        r = requests.get('http://jsonplaceholder.typicode.com/comments', params=payload)
        json = r.json()

        for index in range(len(json)):
            json_item = json[index]
            actual_id = json_item['postId']   
            self.assertEquals(actual_id, expected_post_id)    

    def test_post_json(self): 
        payload = {
            'title': 'tttttiiihihihitle',
            'body': 'aaaaaaaahahaha',
            'userId': 1000
        }

        response = requests.post('http://jsonplaceholder.typicode.com/posts', json=payload)
        
        self.assertEqual(response.status_code, 201)



class Track:
    """Represents a track"""

    def __init__(self, track_name, artist):
        self.track_name = track_name
        self.artist = artist

    @staticmethod
    def are_equivalent(track_1, track_2):
        """Returns true if the given tracks are equivalent (ie: same track name and artist)"""
        return track_1.track_name.casefold() == track_2.track_name.casefold() \
            and track_1.artist.casefold() == track_2.artist.casefold()

    def __eq__(self, other):
        return isinstance(other, Track) \
               and self.track_name == other.track_name \
               and self.artist == other.artist \

    def __repr__(self):
        return str(self.__dict__)



#!/usr/bin/env python3.7

from configparser import ConfigParser
import argparse
import os
from lastipy import definitions
from lastipy.lastfm.library.top_tracks import fetch_top_tracks
from lastipy.lastfm.recommendations.similar_tracks import fetch_similar_tracks
from lastipy.lastfm.recommendations.recommendations import fetch_recommendations
from lastipy.lastfm.library.recent_tracks import fetch_recent_tracks
from lastipy.lastfm.library.recent_artists import fetch_recent_artists
from lastipy.lastfm.library import period
from lastipy.track import Track
from numpy.random import choice
from spotipy import Spotify
from lastipy.spotify import token
from lastipy.util.setup_logging import setup_logging
import logging
from lastipy.util.parse_api_keys import ApiKeysParser
from lastipy.spotify import library, search, playlist


def build_recommendations_playlist():
    """Adds recommendations to the given Spotify user's playlist based on the given Last.fm user's recommendations"""

    setup_logging('recommendations.log')
    args = _extract_args()
    spotify = Spotify(auth=token.get_token(args.spotify_user, args.spotify_client_id_key, args.spotify_client_secret_key))

    recommendations = fetch_recommendations(user=args.lastfm_user,
                                            api_key=args.lastfm_api_key,
                                            recommendation_period=args.recommendation_period,
                                            max_similar_tracks_per_top_track=args.max_recommendations_per_top_track,
                                            blacklisted_artists=args.blacklisted_artists,
                                            prefer_unheard_artists=args.prefer_unheard_artists)
    
    library_saved_tracks = library.get_saved_tracks(spotify)
    library_playlist_tracks = library.get_tracks_in_playlists(spotify)

    tracks_for_playlist = []
    while len(tracks_for_playlist) < args.playlist_size and len(recommendations) > 0:
        recommendation = choice(recommendations, p=_calculate_rating_weights(recommendations))
        recommendations.remove(recommendation)

        search_results = search.search_for_tracks(spotify=spotify,
                                                  query=recommendation.artist + " " + recommendation.track_name)
        # Always use the first result, which we can assume is the closest match
        first_result = search_results[0] if search_results else None

        if first_result is not None \
           and Track.are_equivalent(first_result, recommendation) \
           and first_result not in tracks_for_playlist \
           and first_result not in library_playlist_tracks \
           and first_result not in library_saved_tracks \
           and not any(first_result.artist == item.artist for item in tracks_for_playlist):
            logging.debug("Adding " + str(first_result))
            tracks_for_playlist.append(first_result)

    playlist.replace_tracks_in_playlist(spotify, args.playlist_name, tracks_for_playlist)

    logging.info("Done!")

def _extract_args():
    args_parser = _setup_args_parser()
    args = args_parser.parse_args()
    args = _extract_user_configs(args)
    args = _extract_api_keys(args)
    return args


def _setup_args_parser():
    parser = argparse.ArgumentParser(description="Create a Spotify playlist based off recommendations from Last.fm")
    parser.add_argument('user_configs_file', type=argparse.FileType('r', encoding='UTF-8'))
    parser.add_argument('api_keys_file', type=argparse.FileType('r', encoding='UTF-8'))
    return parser


def _extract_user_configs(args):
    config_parser = ConfigParser()
    config_parser.read(args.user_configs_file.name)
    section = 'Config'
    args.lastfm_user = config_parser[section]['LastFMUser']
    args.spotify_user = config_parser[section]['SpotifyUser']
    args.recommendation_period = config_parser[section]['RecommendationPeriod']
    args.max_recommendations_per_top_track = int(config_parser[section]['MaxRecommendationsPerTopTrack'])
    args.playlist_size = int(config_parser[section]['PlaylistSize'])
    args.playlist_name = config_parser[section]['PlaylistName']
    args.blacklisted_artists = config_parser[section]['BlacklistedArtists'].split(",")
    args.prefer_unheard_artists = _str_to_bool(config_parser[section]['PreferUnheardArtists'])
    return args


def _str_to_bool(to_convert):
    return to_convert == "True"


def _extract_api_keys(args):
    keys_parser = ApiKeysParser(args.api_keys_file)
    args.lastfm_api_key = keys_parser.lastfm_api_key
    args.spotify_client_id_key = keys_parser.spotify_client_id_key
    args.spotify_client_secret_key = keys_parser.spotify_client_secret_key
    return args


def _calculate_rating_weights(recommendations):
    total_ratings = 0
    for recommendation in recommendations:
        total_ratings += recommendation.recommendation_rating

    rating_weights = []
    for recommendation in recommendations:
        rating_weights.append(recommendation.recommendation_rating / total_ratings)
    return rating_weights

if __name__ == "__main__":
    build_recommendations_playlist()


import unittest
from unittest.mock import patch

from lastipy.lastfm.recommendations.recommendations import fetch_recommendations
from lastipy.lastfm.library.top_track import TopTrack
from lastipy.lastfm.recommendations.recommended_track import RecommendedTrack


class RecommendationsFetcherTest(unittest.TestCase):

    @patch('lastipy.lastfm.recommendations.recommendations.fetch_recent_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_top_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_similar_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.calculate_ratings')
    def test_recent_tracks_are_filtered(self, mock_calculate_ratings, mock_similar_tracks, mock_top_tracks, mock_recent_tracks):
        recent_track_1 = TopTrack(track_name="SWALBR", artist="Cream", playcount=1)
        recent_track_2 = TopTrack(track_name="Badge", artist="Cream", playcount=1)
        recent_track_3 = TopTrack(track_name="Layla", artist="Derek & the Dominos", playcount=8)
        mock_recent_tracks.return_value = [recent_track_1, recent_track_2, recent_track_3]

        mock_top_tracks.return_value = [recent_track_3]

        new_recommendation = RecommendedTrack(track_name="Key to the Highway", artist="Derek & the Dominos", recommendation_rating=8)
        already_scrobbled_recommendation_1 = RecommendedTrack(track_name="Badge", artist="Cream", recommendation_rating=8)
        already_scrobbled_recommendation_2 = RecommendedTrack(track_name="SWALBR", artist="Cream", recommendation_rating=8)
        recommendations = [new_recommendation, already_scrobbled_recommendation_1, already_scrobbled_recommendation_2]
        mock_similar_tracks.return_value = recommendations

        mock_calculate_ratings.return_value = recommendations

        self.assertCountEqual(fetch_recommendations('test', ''), [new_recommendation])


    @patch('lastipy.lastfm.recommendations.recommendations.fetch_recent_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_top_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_similar_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.calculate_ratings')
    def test_blacklisted_artists_are_filtered(self, mock_calculate_ratings, mock_similar_tracks, mock_top_tracks, mock_recent_tracks):
        mock_recent_tracks.return_value = []
        mock_top_tracks.return_value = [TopTrack(track_name='Here Comes the Sun', artist='The Beatles', playcount=5)]

        recommendation_1 = RecommendedTrack(track_name="Badge", artist="Cream", recommendation_rating=1)
        recommendation_2 = RecommendedTrack(track_name="Penny Lane", artist="The Beatles", recommendation_rating=1)
        recommendations = [recommendation_1, recommendation_2]
        mock_similar_tracks.return_value = recommendations

        mock_calculate_ratings.return_value = recommendations

        recommendations = fetch_recommendations(user="meeee", api_key='', blacklisted_artists=['The Beatles'])

        self.assertCountEqual(recommendations, [recommendation_1])


    @patch('lastipy.lastfm.recommendations.recommendations.fetch_recent_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_top_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.fetch_similar_tracks')
    @patch('lastipy.lastfm.recommendations.recommendations.calculate_ratings')
    def test_blacklisted_artists_filtering_should_ignore_case(self, mock_calculate_ratings, mock_similar_tracks, mock_top_tracks, mock_recent_tracks):
        mock_recent_tracks.return_value = []
        mock_top_tracks.return_value = [TopTrack(track_name='everything i wanted', artist='Billie Eilish', playcount=5)]

        recommendation = RecommendedTrack(track_name="Bad Music", artist="Zayn", recommendation_rating=1)
        recommendations = [recommendation]
        mock_similar_tracks.return_value = recommendations

        mock_calculate_ratings.return_value = recommendations

        recommendations = fetch_recommendations(user="", api_key='', blacklisted_artists=['ZAYN'])

        self.assertEqual(0, len(recommendations))



import spotipy
from lastipy.spotify.parse_spotify_tracks import parse_tracks
from datetime import datetime
import logging
from lastipy.track import Track


#TODO test
def fetch_new_tracks(spotify, as_of_date=datetime.today().date()):
    """Fetches new tracks (as of the given date) released by the current Spotify user's followed artists"""

    followed_artist_ids = _get_followed_artists(spotify)

    all_albums = []
    for artist_id in followed_artist_ids:
        artist_albums = _get_releases(spotify, artist_id)
        all_albums += artist_albums

    new_albums = _filter_new_releases(all_albums, as_of_date)

    all_tracks = []
    for album in new_albums:
        all_tracks += _get_album_tracks(spotify, album)

    new_tracks = parse_tracks(all_tracks)
    
    new_tracks = _remove_duplicates(new_tracks)
    logging.info("Fetched new tracks " + str(new_tracks))
    return new_tracks


def _get_followed_artists(spotify):
    followed_artists = []

    curr_response = spotify.current_user_followed_artists(limit=50)

    while len(curr_response['artists']['items']) > 0:
        curr_response = spotify.current_user_followed_artists(limit=50, after=curr_response['artists']['items'][len(curr_response) - 1]['id'])
        followed_artists += curr_response['artists']['items']

    # The above Spotipy function doesn't really seem to function properly and results in duplicates, 
    # so we remove them here by converting the list to just the IDs (not doing so results in
    # an unhashable error), then converting to a set and back to a list 
    followed_artists = [artist['id'] for artist in followed_artists]
    followed_artist_ids = list(set(followed_artists))
    return followed_artist_ids


def _filter_new_releases(all_albums, as_of_date):
    new_albums = []
    for album in all_albums:
        if album['release_date_precision'] == 'day':
            if datetime.strptime(album['release_date'], "%Y-%m-%d").date() >= as_of_date:
                new_albums.append(album)
        elif album['release_date_precision'] == 'month':
            release_date = datetime.strptime(album['release_date'], "%Y-%m")
            if release_date.year > as_of_date.year or (release_date.year == as_of_date.year and release_date.month >= as_of_date.month):
                   new_albums.append(album)
        elif album['release_date_precision'] == 'year':
            if datetime.strptime(album['release_date'], '%Y').year >= as_of_date.year:
                new_albums.append(album)
    return new_albums


def _get_releases(spotify, artist_id):
    # Albums
    curr_response = spotify.artist_albums(artist_id, album_type='album', limit=50)
    artist_albums = curr_response['items']
    while len(curr_response['items']) > 0:
        curr_response = spotify.artist_albums(artist_id, album_type='album', limit=50, offset=len(artist_albums))
        artist_albums += curr_response['items']
    # Singles
    curr_response = spotify.artist_albums(artist_id, album_type='single', limit=50)
    artist_singles = curr_response['items']
    while len(curr_response['items']) > 0:
        curr_response = spotify.artist_albums(artist_id, album_type='single', limit=50, offset=len(artist_singles))
        artist_singles += curr_response['items']
    return artist_albums + artist_singles


def _get_album_tracks(spotify, album):
    curr_response = spotify.album_tracks(album['id'], limit=50)
    album_tracks = curr_response['items']
    while len(curr_response['items']) > 0:
        curr_response = spotify.album_tracks(album['id'], limit=50, offset=len(album_tracks))
        album_tracks += curr_response['items']
    return album_tracks


def _remove_duplicates(tracks):
    """Removes duplicate tracks (ie: tracks that have identical name and artist; see Track.are_equivalent)"""
    logging.debug("New tracks before removing duplicates: " + str(tracks))
    tracks_without_duplicates = []
    # We remove duplicates with a list comprehension rather than the traditional hack of using a set, since that
    # requires the object to be hashable; plus we only want to compare the track name/artist of each track, not
    # any of the other fields (eg: Spotify ID) which might in fact differ
    [tracks_without_duplicates.append(track_x) 
        for track_x in tracks 
        if not any(Track.are_equivalent(track_x, track_y) 
                   for track_y in tracks_without_duplicates)]
    logging.debug("New tracks after removing duplicates: " + str(tracks_without_duplicates))
    return tracks_without_duplicates


from lastipy.spotify.parse_spotify_tracks import parse_tracks
import spotipy
import logging


#TODO test
def get_tracks_in_playlists(spotify):
    """Returns all tracks in the currently-logged-in user's playlists"""

    logging.info("Fetching all tracks in " + spotify.current_user()['id'] + "'s playlists")

    playlists = spotify.current_user_playlists()['items']

    tracks = []
    for playlist in playlists:
        tracks = tracks + get_tracks_in_playlist(spotify, playlist_id=playlist['id'])

    logging.info("Fetched tracks " + str(tracks))

    return tracks


def get_tracks_in_playlist(spotify, playlist_id):
    """Returns all tracks in the given playlist"""

    tracks_in_playlist = []

    keep_fetching = True
    while keep_fetching:
        json_response = spotify.playlist_tracks(playlist_id=playlist_id,
                                                offset=len(tracks_in_playlist))
        if json_response['items']:
            tracks_in_playlist = tracks_in_playlist + parse_tracks(json_response['items'])
        else:
            keep_fetching = False

    return tracks_in_playlist


def get_saved_tracks(spotify):
    """Returns the currently-logged-in users's saved tracks"""

    logging.info("Fetching " + spotify.current_user()['id'] + "'s saved tracks")

    saved_tracks = []
    keep_fetching = True
    while keep_fetching:
        json_response = spotify.current_user_saved_tracks(offset=len(saved_tracks))
        if json_response['items']:
            saved_tracks = saved_tracks + parse_tracks(json_response['items'])
        else:
            keep_fetching = False

    logging.info("Fetched tracks: " + str(saved_tracks))

    return saved_tracks


def add_tracks_to_library(spotify, tracks):
    logging.info("Adding " + str(tracks) + " to " + spotify.current_user()['id'] + "'s library")
    # Spotify only allows 50 tracks to be added to a library at once, so we need to chunk 'em up
    limit = 50
    track_chunks = [tracks[i:i + limit] for i in range(0, len(tracks), limit)]  
    for chunk in track_chunks:
        spotify.current_user_saved_tracks_add([track.spotify_id for track in chunk])


import unittest
from unittest.mock import patch

from lastipy.lastfm.library.scrobbled_artist import ScrobbledArtist
from lastipy.lastfm.library.top_track import TopTrack
from lastipy.lastfm.recommendations.rating_calculator import calculate_ratings 
from lastipy.lastfm.recommendations.recommended_track import RecommendedTrack


class RatingCalculatorTest(unittest.TestCase):

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_rating_with_one_top_track_one_recommendation(self, mock_recent_artists):
        mock_recent_artists.return_value = []
        top_track = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=10)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track: [recommended_track_1]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(10, recommendations[0].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_rating_with_one_top_track_multiple_recommendations(self, mock_recent_artists):
        mock_recent_artists.return_value = []
        top_track = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=10)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        recommended_track_2 = RecommendedTrack(artist="The Beatles", track_name='Baby You\'re a Rich Man', recommendation_rating=1)
        recommended_track_3 = RecommendedTrack(artist="The Beatles", track_name="Blue Jay Way", recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track: [recommended_track_1, recommended_track_2, recommended_track_3]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(10, recommendations[0].recommendation_rating)
        self.assertEqual(10, recommendations[1].recommendation_rating)
        self.assertEqual(10, recommendations[2].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_rating_with_multiple_top_tracks_multiple_recommendations(self, mock_recent_artists):
        mock_recent_artists.return_value = []
        top_track_1 = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=10)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        recommended_track_2 = RecommendedTrack(artist="The Beatles", track_name='Baby You\'re a Rich Man', recommendation_rating=1)
        recommended_track_3 = RecommendedTrack(artist="The Beatles", track_name="Blue Jay Way", recommendation_rating=1)
        top_track_2 = TopTrack(artist='The Rolling Stones', track_name='Satisfaction', playcount=20)
        recommended_track_4 = RecommendedTrack(artist='The Rolling Stones', track_name='Ruby Tuesday', recommendation_rating=1)
        recommended_track_5 = RecommendedTrack(artist="The Beatles", track_name='Get Off Of My Cloud', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track_1: [recommended_track_1, recommended_track_2, recommended_track_3],
            top_track_2: [recommended_track_4, recommended_track_5]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(10, recommendations[0].recommendation_rating)
        self.assertEqual(10, recommendations[1].recommendation_rating)
        self.assertEqual(10, recommendations[2].recommendation_rating)
        self.assertEqual(20, recommendations[3].recommendation_rating)
        self.assertEqual(20, recommendations[4].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_rating_is_reduced_based_on_artist_playcount(self, mock_recent_artists):
        mock_recent_artists.return_value = [ScrobbledArtist(artist_name='The Beatles', playcount=9)]
        top_track = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=5)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        recommended_track_2 = RecommendedTrack(artist='The Beatles', track_name='Blue Jay Way', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track: [recommended_track_1, recommended_track_2]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(0.1, recommendations[0].recommendation_rating)
        self.assertEqual(0.1, recommendations[1].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_multiple_recommendations_and_multiple_recent_artists(self, mock_recent_artists):
        mock_recent_artists.return_value = [ScrobbledArtist(artist_name='The Beatles', playcount=9),
                                            ScrobbledArtist(artist_name='The Rolling Stones', playcount=19)]    
        top_track_1 = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=5)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        top_track_2 = TopTrack(artist='The Rolling Stones', track_name='Satisfaction', playcount=10)
        recommended_track_2 = RecommendedTrack(artist='The Kinks', track_name='You Really Got Me', recommendation_rating=1)
        recommended_track_3 = RecommendedTrack(artist='The Rolling Stones', track_name='Ruby Tuesday', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track_1: [recommended_track_1],
            top_track_2: [recommended_track_2, recommended_track_3]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(0.1, recommendations[0].recommendation_rating)
        self.assertEqual(10, recommendations[1].recommendation_rating)
        self.assertEqual(0.1, recommendations[2].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_recommendations_where_artist_has_one_playcount_should_get_rating_halved(self, mock_recent_artists):
        mock_recent_artists.return_value = [ScrobbledArtist(artist_name='The Beatles', playcount=1)]
        top_track = TopTrack(artist='The Beatles', track_name='Penny Lane', playcount=10)
        recommended_track_1 = RecommendedTrack(artist='The Beatles', track_name='Strawberry Fields Forever', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track: [recommended_track_1]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(1, recommendations[0].recommendation_rating)

    @patch('lastipy.lastfm.recommendations.rating_calculator.fetch_recent_artists')
    def test_reducing_based_on_recent_artists_ignores_case(self, mock_recent_artists):
        mock_recent_artists.return_value = [ScrobbledArtist(artist_name='LANY', playcount=3)]
        top_track = TopTrack(artist='LANY', track_name='Some Whiny Crap', playcount=2)
        recommended_track = RecommendedTrack(artist='Lany', track_name='Some Other Whiny Crap', recommendation_rating=1)
        top_tracks_to_recommendations = {
            top_track: [recommended_track]
        }
        recommendations = calculate_ratings('test', '', top_tracks_to_recommendations)
        self.assertEqual(0.1, recommendations[0].recommendation_rating)


import logging, requests
from lastipy.lastfm.library.scrobbled_artist import ScrobbledArtist

URL = 'http://ws.audioscrobbler.com/2.0/?method=library.getartists'
MAX_RESULTS_PER_PAGE = 200


#TODO test
def fetch_recent_artists(user, api_key):
    """Fetches tracks similar to the given track"""

    logging.info("Fetching recent artists for user " + user)
    artists = []
    page = 1
    total_pages = 1
    while page <= total_pages:
        json_response = _send_request(_build_json_payload(user, api_key, page))
        logging.debug("Response: " + str(json_response))
        for artist in json_response['artists']['artist']:
            artists.append(ScrobbledArtist(artist_name=artist['name'], playcount=int(artist['playcount'])))
        total_pages = int(json_response['artists']['@attr']['totalPages'])
        page = page + 1

    logging.info("Fetched recent artists " + str(artists))

    return artists

def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status()

def _build_json_payload(user, api_key, page):
    payload = {
        'format': 'json',
        'page': page,
        'api_key': api_key,
        'limit': MAX_RESULTS_PER_PAGE,
        'user': user
    }
    return payload



from lastipy.spotify.spotify_track import SpotifyTrack

#TODO test
def parse_tracks(json_tracks):
    return [_parse_track(json_track) for json_track in json_tracks]


def _parse_track(json_track):
    if 'track' in json_track:
        # Some endpoints do this, others don't
        json_track = json_track['track']
    track_id = json_track['id']
    name = json_track['name']
    # Just getting the first artist, even if there's multiple
    artist = json_track['artists'][0]['name']
    return SpotifyTrack(track_name=name, artist=artist, spotify_id=track_id)



#!/usr/bin/env python3.7

from configparser import ConfigParser
import argparse
import os
from lastipy import definitions
from lastipy.lastfm.library.top_tracks import fetch_top_tracks
from lastipy.lastfm.recommendations.similar_tracks import fetch_similar_tracks
from lastipy.lastfm.recommendations.recommendations import fetch_recommendations
from lastipy.lastfm.library.recent_tracks import fetch_recent_tracks
from lastipy.lastfm.library.recent_artists import fetch_recent_artists
from lastipy.lastfm.library import period
from lastipy.spotify import playlist, search, library
from lastipy.track import Track
from numpy.random import choice
from spotipy import Spotify
from lastipy.spotify import token
from lastipy.util.setup_logging import setup_logging
import logging
from lastipy.spotify import new_releases
from datetime import datetime
from lastipy.util.parse_api_keys import ApiKeysParser


def save_new_tracks():
    """Saves new tracks (as of the current date) from the specified Spotify user's followed artists to their library"""

    setup_logging("new_releases.log")
    args = _extract_args()
    spotify = Spotify(auth=token.get_token(args.spotify_user, args.spotify_client_id_key, args.spotify_client_secret_key))

    new_tracks = new_releases.fetch_new_tracks(spotify)

    if len(new_tracks) > 0:
        # Only process further if we actually fetched any new tracks; otherwise there's no point

        # Filter out new tracks that are already saved in the library
        tracks_to_save = _filter_out_already_saved_tracks(spotify, new_tracks)

        library.add_tracks_to_library(spotify, tracks_to_save)
    else:
        logging.info("No new tracks to add!")

    logging.info("Done!")

def _filter_out_already_saved_tracks(spotify, new_tracks):
    saved_tracks = library.get_saved_tracks(spotify)
    logging.info("Filtering out already saved tracks from new tracks")
    tracks_to_save = [new_track for new_track in new_tracks 
                        if not any(new_track == saved_track for saved_track in saved_tracks)]
    return tracks_to_save

def _extract_args():
    args = _parse_args()

    # Parse API keys file
    keys_parser = ApiKeysParser(args.api_keys_file)
    args.spotify_client_id_key = keys_parser.spotify_client_id_key
    args.spotify_client_secret_key = keys_parser.spotify_client_secret_key
    
    return args

def _parse_args():
    args_parser = argparse.ArgumentParser(description="Adds new tracks from followed artists to a Spotify playlist")
    args_parser.add_argument('spotify_user', type=str)
    args_parser.add_argument('api_keys_file', type=argparse.FileType('r', encoding='UTF-8'))
    return args_parser.parse_args()

if __name__ == "__main__":
    save_new_tracks()


import logging
import requests
from lastipy.lastfm.parse_lastfm_tracks import parse_artist, parse_track_name
from lastipy.track import Track
from requests import RequestException

URL = 'http://ws.audioscrobbler.com/2.0/?method=user.getrecenttracks'
MAX_RESULTS_PER_PAGE = 200
MAX_RETRIES = 10


#TODO test
def fetch_recent_tracks(user, api_key):
    """Fetches recent tracks for the given user"""

    logging.info("Fetching recent tracks for " + user + "...")
    all_recent_tracks = []
    page = 1
    total_pages = 1
    retries = 0
    while page <= total_pages:
        try:
            json_response = _send_request(_build_json_payload(user, api_key, page))
            logging.debug("Response: " + str(json_response))
            json_tracks = json_response['recenttracks']['track']
            recent_tracks_on_curr_page = [Track(parse_track_name(json_track), parse_artist(json_track))
                                          for json_track in json_tracks]
            all_recent_tracks = all_recent_tracks + recent_tracks_on_curr_page
            total_pages = int(json_response['recenttracks']['@attr']['totalPages'])
            page = page + 1
        except RequestException:
            # This particular endpoint has a habit of throwing back error 500, so just retry if it does
            if retries < MAX_RETRIES:
                logging.warning("Failed to fetch recent tracks page " + str(page) + ". Retrying...")
                retries = retries + 1
            else:
                logging.warning("Failed to fetch recent tracks page " + str(page) +
                                " after " + str(retries) + " retries. Giving up and moving on...")
                break

    logging.info(f"Fetched " + str(len(all_recent_tracks)) + " recent tracks: " + str(all_recent_tracks))
    return all_recent_tracks

def _send_request(json_payload):
    response = requests.get(URL, params=json_payload)
    if response.ok:
        return response.json()
    else:
        response.raise_for_status()

def _build_json_payload(user, api_key, page):
    payload = {
        'user': user,
        'format': 'json',
        'api_key': api_key,
        'limit': MAX_RESULTS_PER_PAGE,
        'page': page
    }
    return payload
